@@ -9,7 +9,7 @@ use Carp qw(croak carp);
 use Data::Dumper;
 use XML::Entities;
 
-use base qw(Exporter);
+use parent qw(Exporter);
 
 @EXPORT_OK = qw(
 	parse_plist
@@ -24,7 +24,9 @@ use base qw(Exporter);
 	'all' => \@EXPORT_OK,
 	);
 
-$VERSION = '1.34';
+$VERSION = '1.412';
+
+=encoding utf8
 
 =head1 NAME
 
@@ -32,9 +34,10 @@ Mac::PropertyList - work with Mac plists at a low level
 
 =head1 SYNOPSIS
 
-	use Mac::PropertyList;
+	use Mac::PropertyList qw(:all);
 
 	my $data  = parse_plist( $text );
+	my $perl  = $data->as_perl;
 
 		# == OR ==
 	my $data  = parse_plist_file( $filename );
@@ -51,13 +54,15 @@ Mac::PropertyList - work with Mac plists at a low level
 
 	my $plist = Mac::PropertyList::dict->new( \%hash );
 
+	my $perl  = $plist->as_perl;
 
 =head1 DESCRIPTION
 
 This module is a low-level interface to the Mac OS X Property List
-(plist) format. You probably shouldn't use this in applications--build
-interfaces on top of this so you don't have to put all the heinous
-multi-level object stuff where people have to look at it.
+(plist) format in either XML or binary. You probably shouldn't use
+this in applications–build interfaces on top of this so you don't have
+to put all the heinous multi-level object stuff where people have to
+look at it.
 
 You can parse a plist file and get back a data structure. You can take
 that data structure and get back the plist as XML. If you want to
@@ -66,6 +71,14 @@ change the structure inbetween that's your business. :)
 You don't need to be on Mac OS X to use this. It simply parses and
 manipulates a text format that Mac OS X uses.
 
+If you need to work with the old ASCII or newer JSON formet, you can
+use the B<plutil> tool that comes with MacOS X:
+
+	% plutil -convert xml1 -o ExampleBinary.xml.plist ExampleBinary.plist
+
+Or, you can extend this module to handle those formats (and send a pull
+request).
+
 =head2 The Property List format
 
 The MacOS X Property List format is simple XML. You can read the DTD
@@ -73,7 +86,7 @@ to get the details.
 
 	http://www.apple.com/DTDs/PropertyList-1.0.dtd
 
-One big problem exists--its dict type uses a flat structure to list
+One big problem exists—its dict type uses a flat structure to list
 keys and values so that values are only associated with their keys by
 their position in the file rather than by the structure of the DTD.
 This problem is the major design hinderance in this module. A smart
@@ -102,6 +115,12 @@ There are several types of objects:
 	Mac::PropertyList::date
 	Mac::PropertyList::array
 	Mac::PropertyList::dict
+	Mac::PropertyList::true
+	Mac::PropertyList::false
+
+Note that the Xcode property list editor abstracts the C<true> and
+C<false> objects as just C<Boolean>. They are separate tags in the
+plist format though.
 
 =over 4
 
@@ -122,6 +141,12 @@ Access the type of the object (string, data, etc)
 
 Create a string version of the object, recursively if necessary.
 
+=item as_perl
+
+Turn the plist data structure, which is decorated with extra
+information, into a lean Perl data structure without the value type
+information or blessed objects.
+
 =back
 
 =cut
@@ -144,7 +169,6 @@ my %Readers = (
 	"date"    => \&read_date,
 	"real"    => \&read_real,
 	"integer" => \&read_integer,
-	"string"  => \&read_string,
 	"array"   => \&read_array,
 	"data"    => \&read_data,
 	"true"    => \&read_true,
@@ -178,6 +202,7 @@ sub parse_plist {
 
 	my $plist = do {
 		if( $text =~ /\A<\?xml/ ) { # XML plists
+			$text =~ s/<!--(?:[\d\D]*?)-->//g;
 			# we can handle either 0.9 or 1.0
 			$text =~ s|^<\?xml.*?>\s*<!DOC.*>\s*<plist.*?>\s*||;
 			$text =~ s|\s*</plist>\s*$||;
@@ -253,7 +278,7 @@ sub parse_plist_file {
 
 Create a plist dictionary from the hash reference.
 
-The values of the hash can only be simple scalars--not references.
+The values of the hash can only be simple scalars–not references.
 Reference values are silently ignored.
 
 Returns a string representing the hash in the plist format.
@@ -289,7 +314,7 @@ sub create_from_hash {
 
 Create a plist array from the array reference.
 
-The values of the array can only be simple scalars--not references.
+The values of the array can only be simple scalars–not references.
 Reference values are silently ignored.
 
 Returns a string representing the array in the plist format.
@@ -357,12 +382,15 @@ sub read_next {
 	while( not defined $value ) {
 		croak "Couldn't read anything!" if $source->eof;
 		$_ .= $source->get_line;
-
-		if( s[^\s* < (string|date|real|integer|data) >
-			   \s*(.*?)\s* </\1> ][]sx ) {
+		if( s[^\s* < (string|date|real|integer|data) > \s*(.*?)\s* </\1> ][]sx ) {
 			$value = $Readers{$1}->( $2 );
 			}
 	    elsif( s[^\s* < (dict|array) > ][]x ) {
+			# We need to put back the unprocessed text if
+			# any because the <dict> and <array> readers
+			# need to see it.
+			$source->put_line( $_ ) if defined $_ && '' ne $_;
+			$_ = '';
 			$value = $Readers{$1}->( $source );
 			}
 	    # these next two are some wierd cases i found in the iPhoto Prefs
@@ -452,8 +480,7 @@ Return the plist data structure as XML in the Mac Property List format.
 
 =cut
 
-sub plist_as_string
-	{
+sub plist_as_string {
 	my $object = CORE::shift;
 
 	my $string = $XML_head;
@@ -468,20 +495,12 @@ sub plist_as_string
 =item plist_as_perl
 
 Return the plist data structure as an unblessed Perl data structure.
-There won't be any C<Mac::PropertyList> objects in the results.
+There won't be any C<Mac::PropertyList> objects in the results. This
+is really just C<as_perl>.
 
 =cut
 
-sub plist_as_perl
-	{
-	my $object = CORE::shift;
-
-	my $string = '$VAR = ';
-
-	$string .= $object->as_perl;
-
-	return $string;
-	}
+sub plist_as_perl { $_[0]->as_perl }
 
 # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
 package Mac::PropertyList::Source;
@@ -495,13 +514,34 @@ sub eof { (not @{$_[0]->{buffer}}) and $_[0]->source_eof }
 sub get_line {
 	my $self = CORE::shift;
 
+# I'm not particularly happy with what I wrote here, but that's why
+# you shouldn't write your own buffering code! I might have left over
+# text in the buffer. This could be stuff a higher level looked at and
+# put back with put_line. If there's text there, grab that.
+#
+# But here's the tricky part. If that next part of the text looks like
+# a "blank" line, grab the next next thing and append that.
+#
+# And, if there's nothing in the buffer, ask for more text from
+# get_source_line. Follow the same rules. IF you get back something that
+# looks like a blank line, ask for another and append it.
+#
+# This means that a returned line might have come partially from the
+# buffer and partially from a fresh read.
+#
+# At some point you should have something that doesn't look like a
+# blank line and the while() will break out. Return what you do.
+#
+# Previously, I wasn't appending to $_ so newlines were disappearing
+# as each next read replaced the value in $_. Yuck.
+
 	local $_ = '';
 	while (defined $_ && /^[\r\n\s]*$/) {
 		if( @{$self->{buffer}} ) {
-			$_ = shift @{$self->{buffer}};
+			$_ .= shift @{$self->{buffer}};
 			}
 		else {
-			$_ = $self->get_source_line;
+			$_ .= $self->get_source_line;
 			}
 		}
 
@@ -547,8 +587,6 @@ sub value {
 sub type { my $r = ref $_[0] ? ref $_[0] : $_[0]; $r =~ s/.*:://; $r; }
 
 sub new {
-	#print STDERR "Got [@_]\n";
-
 	bless $_[1], $_[0]
 	}
 
@@ -642,8 +680,6 @@ package Mac::PropertyList::dict;
 use base qw(Mac::PropertyList::Container);
 
 sub new {
-	#print STDERR Data::Dumper::Dumper( $_[1] );
-
 	$_[0]->SUPER::new( $_[1] );
 	}
 
@@ -708,8 +744,7 @@ sub write {
 	return $string;
 	}
 
-sub as_perl
-	{
+sub as_perl {
 	my $self  = CORE::shift;
 
 	my %dict = map {
@@ -834,7 +869,7 @@ brian d foy, C<< <bdfoy@cpan.org> >>
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright (c) 2004-2011 brian d foy.  All rights reserved.
+Copyright © 2004-2015, brian d foy <bdfoy@cpan.org>. All rights reserved.
 
 This program is free software; you can redistribute it and/or modify
 it under the same terms as Perl itself.
